from __future__ import unicode_literals

import copy
import unittest
from datetime import date, datetime

import pytz
from omniORB import EnumItem, StructBase

from pyfco.recoder import CorbaRecoder, UnsupportedEncodingError, decode_iso_date, decode_iso_datetime, \
    encode_iso_date, encode_iso_datetime
from pyfco.utils import CorbaAssertMixin

try:
    from fred_idl.Registry import IsoDate, IsoDateTime
except ImportError:  # pragma: no cover
    IsoDate = None
    IsoDateTime = None

TEST_ENUM_ITEM = EnumItem("MyEnumItem", 42)


class TestIsoDate(CorbaAssertMixin, unittest.TestCase):
    """Test `decode_iso_date` and `encode_iso_date` functions."""

    @unittest.skipIf(IsoDate is None, "fred_idl.Registry.IsoDate struct is not found.")
    def test_decode(self):  # pragma: no cover
        self.assertEqual(decode_iso_date(IsoDate('1970-02-01')), date(1970, 2, 1))

    @unittest.skipIf(IsoDate is None, "fred_idl.Registry.IsoDate struct is not found.")
    def test_decode_invalid(self):  # pragma: no cover
        self.assertRaises(ValueError, decode_iso_date, IsoDate('1970-02-31'))

    @unittest.skipIf(IsoDate is None, "fred_idl.Registry.IsoDate struct is not found.")
    def test_encode(self):  # pragma: no cover
        self.assertEqual(encode_iso_date(date(1970, 2, 1)), IsoDate('1970-02-01'))

    @unittest.skipIf(IsoDate is not None, "fred_idl.Registry.IsoDate struct is not found.")
    def test_encode_not_implemented(self):  # pragma: no cover
        self.assertRaises(RuntimeError, encode_iso_date, date(1970, 2, 1))


class TestIsoDateTime(CorbaAssertMixin, unittest.TestCase):
    """Test `decode_iso_datetime` and `encode_iso_datetime` functions."""

    @unittest.skipIf(IsoDateTime is None, "fred_idl.Registry.IsoDateTime struct is not found.")
    def test_decode(self):  # pragma: no cover
        self.assertEqual(decode_iso_datetime(IsoDate('1970-02-01T12:14:16Z')),
                         datetime(1970, 2, 1, 12, 14, 16, tzinfo=pytz.utc))

    @unittest.skipIf(IsoDateTime is None, "fred_idl.Registry.IsoDateTime struct is not found.")
    def test_decode_zones(self):  # pragma: no cover
        self.assertEqual(decode_iso_datetime(IsoDate('1970-02-01T12:14:16+0000')),
                         datetime(1970, 2, 1, 12, 14, 16, tzinfo=pytz.utc))
        self.assertEqual(decode_iso_datetime(IsoDate('1970-02-01T12:14:16+00:00')),
                         datetime(1970, 2, 1, 12, 14, 16, tzinfo=pytz.utc))
        self.assertEqual(decode_iso_datetime(IsoDate('1970-02-01T12:14:16-0000')),
                         datetime(1970, 2, 1, 12, 14, 16, tzinfo=pytz.utc))
        self.assertEqual(decode_iso_datetime(IsoDate('1970-02-01T12:14:16-00:00')),
                         datetime(1970, 2, 1, 12, 14, 16, tzinfo=pytz.utc))

        self.assertEqual(decode_iso_datetime(IsoDate('1970-02-01T12:14:16+0005')),
                         datetime(1970, 2, 1, 12, 14, 16, tzinfo=pytz.FixedOffset(5)))
        self.assertEqual(decode_iso_datetime(IsoDate('1970-02-01T12:14:16+0020')),
                         datetime(1970, 2, 1, 12, 14, 16, tzinfo=pytz.FixedOffset(20)))
        self.assertEqual(decode_iso_datetime(IsoDate('1970-02-01T12:14:16+0100')),
                         datetime(1970, 2, 1, 12, 14, 16, tzinfo=pytz.FixedOffset(60)))

        self.assertEqual(decode_iso_datetime(IsoDate('1970-02-01T12:14:16+1000')),
                         datetime(1970, 2, 1, 12, 14, 16, tzinfo=pytz.FixedOffset(600)))
        self.assertEqual(decode_iso_datetime(IsoDate('1970-02-01T12:14:16+10:00')),
                         datetime(1970, 2, 1, 12, 14, 16, tzinfo=pytz.FixedOffset(600)))
        self.assertEqual(decode_iso_datetime(IsoDate('1970-02-01T12:14:16-1000')),
                         datetime(1970, 2, 1, 12, 14, 16, tzinfo=pytz.FixedOffset(-600)))
        self.assertEqual(decode_iso_datetime(IsoDate('1970-02-01T12:14:16-10:00')),
                         datetime(1970, 2, 1, 12, 14, 16, tzinfo=pytz.FixedOffset(-600)))

        self.assertEqual(decode_iso_datetime(IsoDate('1970-02-01T12:14:16+1135')),
                         datetime(1970, 2, 1, 12, 14, 16, tzinfo=pytz.FixedOffset(695)))

    @unittest.skipIf(IsoDateTime is None, "fred_idl.Registry.IsoDateTime struct is not found.")
    def test_decode_microseconds(self):  # pragma: no cover
        self.assertEqual(decode_iso_datetime(IsoDate('1970-02-01T12:14:16.123456Z')),
                         datetime(1970, 2, 1, 12, 14, 16, 123456, tzinfo=pytz.utc))
        self.assertEqual(decode_iso_datetime(IsoDate('1970-02-01T12:14:16.000123Z')),
                         datetime(1970, 2, 1, 12, 14, 16, 123, tzinfo=pytz.utc))
        self.assertEqual(decode_iso_datetime(IsoDate('1970-02-01T12:14:16.000123+0200')),
                         datetime(1970, 2, 1, 12, 14, 16, 123, tzinfo=pytz.FixedOffset(120)))

    @unittest.skipIf(IsoDateTime is None, "fred_idl.Registry.IsoDateTime struct is not found.")
    def test_decode_invalid(self):  # pragma: no cover
        self.assertRaises(ValueError, decode_iso_datetime, IsoDateTime('1970-02-31T04:06:08Z'))
        self.assertRaises(ValueError, decode_iso_datetime, IsoDateTime('1970-02-03T04:06:08.666Z'))
        # Datetime without a zone
        self.assertRaises(ValueError, decode_iso_datetime, IsoDateTime('1970-02-03T04:06:08'))

    @unittest.skipIf(IsoDateTime is None, "fred_idl.Registry.IsoDateTime struct is not found.")
    def test_encode(self):  # pragma: no cover
        self.assertEqual(encode_iso_datetime(datetime(1970, 2, 1, 12, 14, 16, tzinfo=pytz.utc)),
                         IsoDateTime('1970-02-01T12:14:16+00:00'))
        self.assertEqual(encode_iso_datetime(datetime(1970, 2, 1, 12, 14, 16, 123456, tzinfo=pytz.utc)),
                         IsoDateTime('1970-02-01T12:14:16.123456+00:00'))
        self.assertEqual(encode_iso_datetime(datetime(1970, 2, 1, 12, 14, 16, 123456, tzinfo=pytz.FixedOffset(90))),
                         IsoDateTime('1970-02-01T12:14:16.123456+01:30'))

    @unittest.skipIf(IsoDateTime is not None, "fred_idl.Registry.IsoDateTime struct is not found.")
    def test_encode_not_implemented(self):  # pragma: no cover
        self.assertRaises(RuntimeError, encode_iso_datetime, datetime(1970, 2, 1, 12, 14, 16, tzinfo=pytz.utc))

    @unittest.skipIf(IsoDateTime is not None, "fred_idl.Registry.IsoDateTime struct is not found.")
    def test_encode_naive(self):  # pragma: no cover
        self.assertRaises(ValueError, encode_iso_datetime, datetime(1970, 2, 1, 12, 14, 16))


class SampleCorbaStruct(StructBase):
    _NP_RepositoryId = "IDL:SampleCorbaStruct:1.0"

    def __init__(self, id, ico, dic, varSymb, vat, handle, name,
                 organization, street1, street2, street3, city,
                 stateorprovince, postalcode, country, telephone,
                 fax, email, url, credit, unspec_credit, access, zones, hidden):
        self.id = id
        self.ico = ico
        self.dic = dic
        self.varSymb = varSymb
        self.vat = vat
        self.handle = handle
        self.name = name
        self.organization = organization
        self.street1 = street1
        self.street2 = street2
        self.street3 = street3
        self.city = city
        self.stateorprovince = stateorprovince
        self.postalcode = postalcode
        self.country = country
        self.telephone = telephone
        self.fax = fax
        self.email = email
        self.url = url
        self.credit = credit
        self.unspec_credit = unspec_credit
        self.access = access
        self.zones = zones
        self.hidden = hidden


class NodeStruct(StructBase):
    """
    Simple corba structure for testing nested structures.
    """

    def __init__(self, text, inner):
        self.text = text
        self.inner = inner


class TestCorbaRecoder(unittest.TestCase):

    recoder_class = CorbaRecoder

    def test_create(self):
        """ CorbaRecoder is created with supported encoding. """
        rec = self.recoder_class("utf-8")
        self.assertIsNotNone(rec)

    def test_create_wrong_encoding(self):
        """ CorbaRecoder raises error for unsupported encoding. """
        self.assertRaises(UnsupportedEncodingError, self.recoder_class, "invalid coding")

    def test_decode_basic_types(self):
        """ CorbaRecoder decodes basic types to python correctly . """
        rec = self.recoder_class("utf-8")

        decoded_str = rec.decode(b'string')
        expected_str = 'string'

        decoded_str_empty = rec.decode(b'')
        expected_str_empty = ''

        decoded_unicode = rec.decode('test \u010d\u0165')
        expected_unicode = 'test \u010d\u0165'

        decoded_str_chars = rec.decode(b'test \xc4\x8d\xc5\xa5')
        expected_str_chars = 'test \u010d\u0165'

        decoded_none = rec.decode(None)
        expected_none = None

        decoded_int = rec.decode(6)
        expected_int = 6

        decoded_int_zero = rec.decode(0)
        expected_int_zero = 0

        decoded_float = rec.decode(6.6)
        expected_float = 6.6

        decoded_float_zero = rec.decode(0.0)
        expected_float_zero = 0.0

        decoded_bool = rec.decode(True)
        expected_bool = True

        decoded_bool2 = rec.decode(False)
        expected_bool2 = False

        self.assertEqual(decoded_str, expected_str)
        self.assertEqual(type(decoded_str), type(expected_str))
        self.assertEqual(decoded_str_empty, expected_str_empty)
        self.assertEqual(type(decoded_str_empty), type(expected_str_empty))
        self.assertEqual(decoded_str_chars, expected_str_chars)
        self.assertEqual(type(decoded_str_chars), type(expected_str_chars))

        self.assertEqual(decoded_unicode, expected_unicode)
        self.assertEqual(type(decoded_unicode), type(expected_unicode))

        self.assertEqual(decoded_none, expected_none)
        self.assertEqual(type(decoded_none), type(expected_none))

        self.assertEqual(decoded_int, expected_int)
        self.assertEqual(type(decoded_int), type(expected_int))
        self.assertEqual(decoded_int_zero, expected_int_zero)
        self.assertEqual(type(decoded_int_zero), type(expected_int_zero))

        self.assertEqual(decoded_float, expected_float)
        self.assertEqual(type(decoded_float), type(expected_float))
        self.assertEqual(decoded_float_zero, expected_float_zero)
        self.assertEqual(type(decoded_float_zero), type(expected_float_zero))

        self.assertEqual(decoded_bool, expected_bool)
        self.assertEqual(type(decoded_bool), type(expected_bool))
        self.assertEqual(decoded_bool2, expected_bool2)
        self.assertEqual(type(decoded_bool2), type(expected_bool2))

    def test_encode_basic_types(self):
        """ CorbaRecoder encodes basic types to corba correctly . """
        rec = self.recoder_class("utf-8")

        encoded_str = rec.encode('string')
        expected_str = b'string'

        encoded_str_empty = rec.encode('')
        expected_str_empty = b''

        encoded_bytes = rec.encode(b'unicode')
        expected_bytes = b'unicode'

        encoded_unicode_chars = rec.encode('test \u010d\u0165')
        expected_unicode_chars = b'test \xc4\x8d\xc5\xa5'

        encoded_none = rec.encode(None)
        expected_none = None

        encoded_int = rec.encode(6)
        expected_int = 6

        encoded_int_zero = rec.encode(0)
        expected_int_zero = 0

        encoded_float = rec.encode(6.6)
        expected_float = 6.6

        encoded_float_zero = rec.encode(0.0)
        expected_float_zero = 0.0

        encoded_bool = rec.encode(True)
        expected_bool = True

        encoded_bool2 = rec.encode(False)
        expected_bool2 = False

        self.assertEqual(encoded_str, expected_str)
        self.assertEqual(type(encoded_str), type(expected_str))
        self.assertEqual(encoded_str_empty, expected_str_empty)
        self.assertEqual(type(encoded_str_empty), type(expected_str_empty))

        self.assertEqual(encoded_bytes, expected_bytes)
        self.assertEqual(type(encoded_bytes), type(expected_bytes))
        self.assertEqual(encoded_unicode_chars, expected_unicode_chars)
        self.assertEqual(type(encoded_unicode_chars), type(expected_unicode_chars))

        self.assertEqual(encoded_none, expected_none)
        self.assertEqual(type(encoded_none), type(expected_none))

        self.assertEqual(encoded_int, expected_int)
        self.assertEqual(type(encoded_int), type(expected_int))
        self.assertEqual(encoded_int_zero, expected_int_zero)
        self.assertEqual(type(encoded_int_zero), type(expected_int_zero))

        self.assertEqual(encoded_float, expected_float)
        self.assertEqual(type(encoded_float), type(expected_float))
        self.assertEqual(encoded_float_zero, expected_float_zero)
        self.assertEqual(type(encoded_float_zero), type(expected_float_zero))

        self.assertEqual(encoded_bool, expected_bool)
        self.assertEqual(type(encoded_bool), type(expected_bool))
        self.assertEqual(encoded_bool2, expected_bool2)
        self.assertEqual(type(encoded_bool2), type(expected_bool2))

    def test_decode_iter_types(self):
        """ CorbaRecoder decodes iter types to python correctly . """
        rec = self.recoder_class("utf-8")

        original_tuple = (b'string', b'unicode', (b'tuple', 4), None, True)
        original_tuple_copy = copy.deepcopy(original_tuple)
        decoded_tuple = rec.decode(original_tuple)
        expected_tuple = ('string', 'unicode', ('tuple', 4), None, True)

        original_list = [b'string', b'unicode', [b'list', 4], None, True]
        original_list_copy = copy.deepcopy(original_list)
        decoded_list = rec.decode(original_list)
        expected_list = ['string', 'unicode', ['list', 4], None, True]

        self.assertEqual(original_tuple, original_tuple_copy)
        self.assertEqual(type(original_tuple), type(original_tuple_copy))
        self.assertEqual(original_tuple[2][0], original_tuple_copy[2][0])
        self.assertEqual(type(original_tuple[2][0]), type(original_tuple_copy[2][0]))

        self.assertEqual(decoded_tuple, expected_tuple)
        self.assertEqual(type(decoded_tuple), type(expected_tuple))
        self.assertEqual(decoded_tuple[2][0], expected_tuple[2][0])
        self.assertEqual(type(decoded_tuple[2][0]), type(expected_tuple[2][0]))

        self.assertEqual(original_list, original_list_copy)
        self.assertEqual(type(original_list), type(original_list_copy))
        self.assertEqual(original_list[2][0], original_list_copy[2][0])
        self.assertEqual(type(original_list[2][0]), type(original_list_copy[2][0]))

        self.assertEqual(decoded_list, expected_list)
        self.assertEqual(type(decoded_list), type(expected_list))
        self.assertEqual(decoded_list[2][0], expected_list[2][0])
        self.assertEqual(type(decoded_list[2][0]), type(expected_list[2][0]))

    def test_encode_iter_types(self):
        """ CorbaRecoder encodes iter types to corba correctly . """
        rec = self.recoder_class("utf-8")

        original_tuple = ('string', 'unicode', ('tuple', 4), None, True)
        original_tuple_copy = copy.deepcopy(original_tuple)
        encoded_tuple = rec.encode(('string', 'unicode', ('tuple', 4), None, True))
        expected_tuple = (b'string', b'unicode', (b'tuple', 4), None, True)

        original_list = ['string', 'unicode', ['list', 4], None, True]
        original_list_copy = copy.deepcopy(original_list)
        encoded_list = rec.encode(['string', 'unicode', ['list', 4], None, True])
        expected_list = [b'string', b'unicode', [b'list', 4], None, True]

        self.assertEqual(original_tuple, original_tuple_copy)
        self.assertEqual(type(original_tuple), type(original_tuple_copy))
        self.assertEqual(original_tuple[2][0], original_tuple_copy[2][0])
        self.assertEqual(type(original_tuple[2][0]), type(original_tuple_copy[2][0]))

        self.assertEqual(encoded_tuple, expected_tuple)
        self.assertEqual(type(encoded_tuple), type(expected_tuple))
        self.assertEqual(encoded_tuple[2][0], expected_tuple[2][0])
        self.assertEqual(type(encoded_tuple[2][0]), type(expected_tuple[2][0]))

        self.assertEqual(original_list, original_list_copy)
        self.assertEqual(type(original_list), type(original_list_copy))
        self.assertEqual(original_list[2][0], original_list_copy[2][0])
        self.assertEqual(type(original_list[2][0]), type(original_list_copy[2][0]))

        self.assertEqual(encoded_list, expected_list)
        self.assertEqual(type(encoded_list), type(expected_list))
        self.assertEqual(encoded_list[2][0], expected_list[2][0])
        self.assertEqual(type(encoded_list[2][0]), type(expected_list[2][0]))

    def test_decode_enum_item(self):
        recoder = self.recoder_class("utf-8")
        self.assertEqual(recoder.decode(TEST_ENUM_ITEM), TEST_ENUM_ITEM)

    def test_encode_enum_item(self):
        recoder = self.recoder_class("utf-8")
        self.assertEqual(recoder.encode(TEST_ENUM_ITEM), TEST_ENUM_ITEM)

    def test_decode_struct(self):
        """ CorbaRecoder decodes corba entity to python correctly . """
        rec = self.recoder_class("utf-8")
        reg = SampleCorbaStruct(
            id=19, ico=b'', dic=b'', varSymb=b'', vat=False,
            handle=b'NEW REG', name=b'name 1', organization=b'',
            street1=b'chars \xc4\x8d\xc5\xa5', street2=b'', street3=b'', city=b'', stateorprovince=b'state',
            postalcode=b'', country=b'CZ', telephone=b'', fax=b'', email=b'', url=b'',
            credit=b'0.00', unspec_credit=b'0.00', access=[], zones=[], hidden=False)
        reg_copy = copy.deepcopy(reg)
        expected = SampleCorbaStruct(
            id=19, ico='', dic='', varSymb='', vat=False,
            handle='NEW REG', name='name 1', organization='',
            street1='chars \u010d\u0165', street2='', street3='', city='', stateorprovince='state',
            postalcode='', country='CZ', telephone='', fax='', email='', url='',
            credit='0.00', unspec_credit='0.00', access=[], zones=[], hidden=False)

        decoded_reg = rec.decode(reg)

        self.assertEqual(reg.__dict__, reg_copy.__dict__)
        self.assertEqual(type(reg.__dict__['street1']), type(reg_copy.__dict__['street1']))
        self.assertEqual(type(reg.__dict__['vat']), type(reg_copy.__dict__['vat']))
        self.assertEqual(type(reg.__dict__['access']), type(reg_copy.__dict__['access']))

        self.assertEqual(decoded_reg.__dict__, expected.__dict__)
        self.assertEqual(type(decoded_reg.__dict__['street1']), type(expected.__dict__['street1']))
        self.assertEqual(type(decoded_reg.__dict__['vat']), type(expected.__dict__['vat']))
        self.assertEqual(type(decoded_reg.__dict__['access']), type(expected.__dict__['access']))

    def test_encode_struct(self):
        """ CorbaRecoder encodes python entity to corba correctly """
        rec = self.recoder_class("utf-8")
        reg = SampleCorbaStruct(
            id=19, ico='', dic='', varSymb='', vat=False,
            handle='NEW REG', name='name 1', organization='',
            street1='chars \u010d\u0165', street2='', street3='', city='', stateorprovince='state',
            postalcode='', country='CZ', telephone='', fax='', email='', url='',
            credit='0.00', unspec_credit='0.00', access=[], zones=[], hidden=False)
        reg_copy = copy.deepcopy(reg)
        expected = SampleCorbaStruct(
            id=19, ico=b'', dic=b'', varSymb=b'', vat=False,
            handle=b'NEW REG', name=b'name 1', organization=b'',
            street1=b'chars \xc4\x8d\xc5\xa5', street2=b'', street3=b'', city=b'', stateorprovince=b'state',
            postalcode=b'', country=b'CZ', telephone=b'', fax=b'', email=b'', url=b'',
            credit=b'0.00', unspec_credit=b'0.00', access=[], zones=[], hidden=False)

        encoded_entity = rec.encode(reg)

        self.assertEqual(reg.__dict__, reg_copy.__dict__)
        self.assertEqual(type(reg.__dict__['street1']), type(reg_copy.__dict__['street1']))
        self.assertEqual(type(reg.__dict__['vat']), type(reg_copy.__dict__['vat']))
        self.assertEqual(type(reg.__dict__['access']), type(reg_copy.__dict__['access']))

        self.assertEqual(encoded_entity.__dict__, expected.__dict__)
        self.assertEqual(type(encoded_entity.__dict__['street1']), type(expected.__dict__['street1']))
        self.assertEqual(type(encoded_entity.__dict__['vat']), type(expected.__dict__['vat']))
        self.assertEqual(type(encoded_entity.__dict__['access']), type(expected.__dict__['access']))

    def test_decode_nested_struct(self):
        rec = self.recoder_class("utf-8")
        obj = NodeStruct(b'A', NodeStruct(b'B', NodeStruct(b'C', None)))

        output = rec.decode(obj)

        # Check output
        self.assertEqual(output.text, 'A')
        self.assertIsInstance(output.text, unicode)
        self.assertIsInstance(output.inner, NodeStruct)
        self.assertEqual(output.inner.text, 'B')
        self.assertIsInstance(output.inner.text, unicode)
        self.assertIsInstance(output.inner.inner, NodeStruct)
        self.assertEqual(output.inner.inner.text, 'C')
        self.assertIsInstance(output.inner.inner.text, unicode)
        self.assertIsNone(output.inner.inner.inner)

        # Check original object
        self.assertEqual(obj.text, 'A')
        self.assertIsInstance(obj.text, str)
        self.assertIsInstance(obj.inner, NodeStruct)
        self.assertEqual(obj.inner.text, 'B')
        self.assertIsInstance(obj.inner.text, str)
        self.assertIsInstance(obj.inner.inner, NodeStruct)
        self.assertEqual(obj.inner.inner.text, 'C')
        self.assertIsInstance(obj.inner.inner.text, str)
        self.assertIsNone(obj.inner.inner.inner)

    def test_encode_nested_struct(self):
        rec = self.recoder_class("utf-8")
        obj = NodeStruct('A', NodeStruct('B', NodeStruct('C', None)))

        output = rec.encode(obj)

        # Check output
        self.assertEqual(output.text, 'A')
        self.assertIsInstance(output.text, str)
        self.assertIsInstance(output.inner, NodeStruct)
        self.assertEqual(output.inner.text, 'B')
        self.assertIsInstance(output.inner.text, str)
        self.assertIsInstance(output.inner.inner, NodeStruct)
        self.assertEqual(output.inner.inner.text, 'C')
        self.assertIsInstance(output.inner.inner.text, str)
        self.assertIsNone(output.inner.inner.inner)

        # Check original object
        self.assertEqual(obj.text, 'A')
        self.assertIsInstance(obj.text, unicode)
        self.assertIsInstance(obj.inner, NodeStruct)
        self.assertEqual(obj.inner.text, 'B')
        self.assertIsInstance(obj.inner.text, unicode)
        self.assertIsInstance(obj.inner.inner, NodeStruct)
        self.assertEqual(obj.inner.inner.text, 'C')
        self.assertIsInstance(obj.inner.inner.text, unicode)
        self.assertIsNone(obj.inner.inner.inner)

    def test_decode_other(self):
        """ Decoding object raise error """
        rec = self.recoder_class("utf-8")
        reg = object()
        self.assertRaises(ValueError, rec.decode, reg)

    def test_encode_other(self):
        """ Encoding object raise error """
        rec = self.recoder_class("utf-8")
        reg = object()
        self.assertRaises(ValueError, rec.encode, reg)

    def test_sanity_dec_enc(self):
        """ encode(decode(obj)) is equal to obj. """
        rec = self.recoder_class("utf-8")
        reg = SampleCorbaStruct(
            id=19, ico=b'', dic=b'', varSymb=b'', vat=False,
            handle=b'NEW REG', name=b'name 1', organization=b'',
            street1=b'chars \xc4\x8d\xc5\xa5', street2=b'', street3=b'', city=b'', stateorprovince=b'state',
            postalcode=b'', country=b'CZ', telephone=b'', fax=b'', email=b'', url=b'',
            credit=b'0.00', unspec_credit=b'0.00', access=[], zones=[], hidden=False)
        expected = SampleCorbaStruct(
            id=19, ico=b'', dic=b'', varSymb=b'', vat=False,
            handle=b'NEW REG', name=b'name 1', organization=b'',
            street1=b'chars \xc4\x8d\xc5\xa5', street2=b'', street3=b'', city=b'', stateorprovince=b'state',
            postalcode=b'', country=b'CZ', telephone=b'', fax=b'', email=b'', url=b'',
            credit=b'0.00', unspec_credit=b'0.00', access=[], zones=[], hidden=False)

        res = rec.encode(rec.decode(reg))

        self.assertEqual(res.__dict__, expected.__dict__)
        self.assertEqual(type(res.__dict__['street1']), type(expected.__dict__['street1']))
        self.assertEqual(type(res.__dict__['vat']), type(expected.__dict__['vat']))
        self.assertEqual(type(res.__dict__['access']), type(expected.__dict__['access']))

    def test_sanity_enc_dec(self):
        """ decode(encode(obj)) has equal types to obj. """
        rec = self.recoder_class("utf-8")
        reg = SampleCorbaStruct(
            id=19, ico='', dic='', varSymb='', vat=False,
            handle='NEW REG', name='name 1', organization='',
            street1='chars \u010d\u0165', street2='', street3='', city='', stateorprovince='state',
            postalcode='', country='CZ', telephone='', fax='', email='', url='',
            credit='0.00', unspec_credit='0.00', access=[], zones=[], hidden=False)
        expected = SampleCorbaStruct(
            id=19, ico='', dic='', varSymb='', vat=False,
            handle='NEW REG', name='name 1', organization='',
            street1='chars \u010d\u0165', street2='', street3='', city='', stateorprovince='state',
            postalcode='', country='CZ', telephone='', fax='', email='', url='',
            credit='0.00', unspec_credit='0.00', access=[], zones=[], hidden=False)

        res = rec.decode(rec.encode(reg))

        self.assertEqual(res.__dict__, expected.__dict__)
        self.assertEqual(type(res.__dict__['street1']), type(expected.__dict__['street1']))
        self.assertEqual(type(res.__dict__['vat']), type(expected.__dict__['vat']))
        self.assertEqual(type(res.__dict__['access']), type(expected.__dict__['access']))
